"""Helper re-exports for CLI behaviors."""

from __future__ import annotations

from revo_module_installer.cli_support import (
    executor_apply as cli_executor_apply,
    plans as cli_plans,
    prompts as cli_prompts,
    search as cli_search,
    tables as cli_tables,
)

InstallPlan = cli_plans.InstallPlan
InstallPlanEntry = cli_plans.InstallPlanEntry
build_install_plan = cli_plans.build_install_plan
default_plan_path = cli_plans.default_plan_path
resolve_manifest_index = cli_plans.resolve_manifest_index
resolve_manifest_source_url = cli_plans.resolve_manifest_source_url
write_install_plan = cli_plans.write_install_plan

apply_manifest_entries = cli_executor_apply.apply_manifest_entries

confirm_module_installation = cli_prompts.confirm_module_installation
prompt_for_module_name = cli_prompts.prompt_for_module_name
prompt_for_search_query = cli_prompts.prompt_for_search_query
prompt_for_variant_id = cli_prompts.prompt_for_variant_id

filter_registry_entries = cli_search.filter_registry_entries
find_registry_entry = cli_search.find_registry_entry
find_registry_entry_by_name = cli_search.find_registry_entry_by_name
format_search_results = cli_search.format_search_results

build_registry_table = cli_tables.build_registry_table
build_search_table = cli_tables.build_search_table
render_manifest_table = cli_tables.render_manifest_table
